/************************************************************************
* Copyright (c) 1995 Valentin Shopov, Blagoevgrad, Bulgaria		*
*									*
* Permission to use, copy, modify, and distribute this material for any	*
* purpose and without fee is hereby granted, provided that the above	*
* copyright notice and this permission notice appear in all copies, and	*
* that the name of the author not be used in advertising or publicity 	*
* pertaining to this material without the specific, prior written 	*
* permission of the author.						*
*									*
* THE AUTHOR MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY	*  
* OF THIS MATERIAL FOR ANY PURPOSE. IT IS PROVIDED "AS IS", WITHOUT ANY	* 
* EXPRESS OR IMPLIED WARRANTIES.					*
*************************************************************************/

/**************************ciilib1.2.2***********************************
 *                                                                      *
 *        Character to Inline Image Conversion Library (CIILIB)         *
 *                                                                      *
 *    JIS-X-0208-XXXX 16x16 BDF Font to CIILIB XBM source convertor.	*
 *              	by Valentin Shopov		                *
 *			valentin@rs.aubg.bg				*
 ************************************************************************/





#include <stdio.h>
#include <string.h>

char curcol[2048], currow[2048];
int KANIND[94 * 94];

char *
getcol (row, numcol)
     char *row;
     int numcol;
{
  int i, j, k;
  char *tmpcol;
  if (numcol <= 0)
    return row;
  tmpcol = curcol;
  for (i = 0, j = 0, k = 0; row[i] != '\n'; i++)
    {
      if (row[i] == ' ' || row[i] == '\t')
	{
	  if (j)
	    {
	      tmpcol[j] = '\0';
	      j = 0;
	      if (++k == numcol)
		return tmpcol;
	    }
	  continue;
	}
      tmpcol[j++] = row[i];
    }
  if (k == numcol - 1)
    {
      tmpcol[j] = '\0';
    }
  else
    tmpcol[0] = '\0';
  return tmpcol;
}

mirror (src)
     char *src;
{
  int bmrow, i, lbit = 0x0001, hbit = 0x0080, hlbit = 0x0100, hhbit = 0x8000,
    dst;
  sscanf (src, "%04x", &bmrow);
  for (i = 0, dst = 0; i < 8; i++)
    {
      dst |= (lbit & bmrow) ? hbit : 0;
      dst |= (hlbit & bmrow) ? hhbit : 0;
      lbit <<= 1;
      hbit >>= 1;
      hlbit <<= 1;
      hhbit >>= 1;
    }
  return dst;
}


main (ac, av)
     char *av[];
{
  int XBMPOS = 0, NOFCH = 0, NOFB, EUC, ROW, KINROW, BMB = 0, cnvtb, x;
  int width = 0, height = 0;
  char *row, *chset, dstfname[32];
  FILE *dstf, *srcf;

  chset = av[ac-1];
  if (ac != 2 && ac != 3 ||
      (strcmp(chset, "JIS") &&
       strcmp(chset, "KSC") &&
       strcmp(chset, "GB") &&
       strcmp(chset, "BIG5")) )
    {
      fprintf (stderr, "Usage: %s <bdf-file> charset\n", av[0]);
      fprintf (stderr, "       bdf-file: JISX0208/KSC5601/GB2312/BIG5 14x14/16x16 bdf font\n");
      fprintf (stderr, "       charset:  JIS/KSC/GB/BIG5\n");
      exit (1);
    }

  row = currow;
  sprintf(dstfname, "_xbmraw_%s.c", chset);
  printf ("Creating \"%s\"...\n", dstfname);
  for (x = 0; x < 94 * 94; x++)
    {
      KANIND[x] = -1;
    }
  dstf = fopen (dstfname, "w+");
  if (dstf == NULL)
    {
      fprintf (stderr, "Error opening \"%s\"!", dstfname);
      exit (1);
    }
  if (ac == 3)
    {
      srcf = fopen (av[1], "r");
      if (srcf == NULL)
	{
	  fprintf (stderr, "Error opening \"%s\"!", av[1]);
	  exit (1);
	}
    }

  fprintf (dstf, "char *xbmsrc_%s[] = {\n", chset);
  for (; ac == 3 && fgets (row, 2048, srcf) != NULL;)
    {
      if (!XBMPOS)
	{

	  if (!strcmp ("FONTBOUNDINGBOX", getcol (row, 1)))
	    {
	      width = atoi (getcol (row, 2));
	      height = atoi (getcol (row, 3));
	      if (! (width == height && (width == 14 || width == 16)) )
		{
		  printf ("This BDF font is not 14x14 or 16x16 size !!!\n");
		  exit (1);
		}
	    }

	  if (!strcmp ("CHARS", getcol (row, 1)))
	    {
	      NOFCH = atoi (getcol (row, 2));
	    }

	}

      if (!strcmp ("ENCODING", getcol (row, 1)))
	{
	  NOFB = 0;
	  EUC = atoi (getcol (row, 2));
	  if (EUC < 40000) EUC += 32896;
	  fprintf (dstf, "/* %x EUC */\n\"\\\n", EUC);
	  ROW = EUC / 256 - 161;
	  KINROW = EUC % 256 - 161;
	  KANIND[ROW * 94 + KINROW] = XBMPOS++;
	}

      if (!strcmp ("ENDCHAR", getcol (row, 1)))
	{
	  BMB = 0;
	  fprintf (dstf, "\",\n");
	  if ((XBMPOS/50)*50 == XBMPOS || XBMPOS == NOFCH)
	    printf ("Completed...  %d%%\r", (XBMPOS * 100) / NOFCH);
	}


      if (BMB)
	{
	  if (NOFB)
	    {
	      fprintf (dstf, "\\\n");
	    }
	  cnvtb = mirror (getcol (row, 1));
	  fprintf (dstf, "\\%03o\\%03o", cnvtb / 256, cnvtb % 256);
	  NOFB = 1;
	}

      if (!strcmp ("BITMAP", getcol (row, 1)))
	BMB = 1;

    }

  fprintf (dstf, "0};\n");
  fprintf (dstf, "int xbmind_%s[] = {\n", chset);
  for (x = 0; ac == 3 && x < 94 * 94; x++)
    {
      fprintf (dstf, "%d,", KANIND[x]);
      if (x % 10 == 9)
	fprintf (dstf, "\n");
    }
  fprintf (dstf, "0};\n");
  fprintf (dstf, "int xbm%sw = %d;\n", chset, width);
  fprintf (dstf, "int xbm%sh = %d;\n", chset, height);
  fclose (dstf);
  if (ac == 3) {
    fclose (srcf);
    if (NOFCH != XBMPOS)
      {
	printf ("\nSomething wrong! Please check your BDF file!\n");
	exit (1);
      }
  }
  printf ("\nNumber of defined chars: %d\nNumber of processed chars: %d\nDone with creating \"%s\".\n", NOFCH, XBMPOS, dstfname);
  exit (0);
}
